@@ -21,16 +21,20 @@
    
 *******************************************************************************/
 
-#include <boost/python.hpp>
-#include <tango/tango.h>
+#include "precompiled_header.hpp"
+#include <tango.h>
 #include "tango_numpy.h"
 #include "device_attribute.h"
 
 using namespace boost::python;
 
+const int i = 1;
+#define IS_BIGENDIAN() ( (*(char*)&i) == 0 )
+
 namespace PyEncodedAttribute
 {
-    /// This callback is run to delete char* objects.
+
+	/// This callback is run to delete char* objects.
     /// It is called by python. The array was associated with an attribute
     /// value object that is not being used anymore.
     /// @param ptr_ The array object.
@@ -44,7 +48,7 @@ namespace PyEncodedAttribute
         else if (2 == t)
             delete [] (static_cast<unsigned short*>(ptr_));
         else if (4 == t)
-            delete [] (static_cast<uint32_t*>(ptr_));
+			delete [] (static_cast<Tango::DevULong*>(ptr_));
     }
     
     void encode_gray8(Tango::EncodedAttribute &self, object py_value, int w, int h)
@@ -57,6 +61,7 @@ namespace PyEncodedAttribute
             self.encode_gray8(buffer, w, h);
             return;
         }
+#ifndef DISABLE_PYTANGO_NUMPY
         else if (PyArray_Check(py_value_ptr))
         {
             w = PyArray_DIM(py_value_ptr, 1);
@@ -66,11 +71,14 @@ namespace PyEncodedAttribute
             self.encode_gray8(buffer, w, h);
             return;
         }
+#endif
         // It must be a py sequence
         // we are sure that w and h are given by python (see encoded_attribute.py)
-        unsigned char b[w*h];
-        buffer = b;
-        unsigned char *p = b;
+		const int length = w*h;
+	    unsigned char *raw_b = new unsigned char[length];
+		auto_ptr<unsigned char> b(raw_b);
+        buffer = raw_b;
+        unsigned char *p = raw_b;
         int w_bytes = w;
         for (long y=0; y<h; ++y)
         {
@@ -131,7 +139,7 @@ namespace PyEncodedAttribute
                     else if (PyInt_Check(cell) || PyLong_Check(cell))
                     {
                         long byte = PyLong_AsLong(cell);
-                        if (byte==-1 and PyErr_Occurred())
+                        if (byte==-1 && PyErr_Occurred())
                         {
                             Py_DECREF(row);
                             Py_DECREF(cell);
@@ -167,6 +175,7 @@ namespace PyEncodedAttribute
             self.encode_jpeg_gray8(buffer, w, h, quality);
             return;
         }
+#ifndef DISABLE_PYTANGO_NUMPY
         else if (PyArray_Check(py_value_ptr))
         {
             w = PyArray_DIM(py_value_ptr, 1);
@@ -176,11 +185,14 @@ namespace PyEncodedAttribute
             self.encode_jpeg_gray8(buffer, w, h, quality);
             return;
         }
+#endif
         // It must be a py sequence
         // we are sure that w and h are given by python (see encoded_attribute.py)
-        unsigned char b[w*h];
-        buffer = b;
-        unsigned char *p = b;
+		const int length = w*h;
+        unsigned char *raw_b = new unsigned char[length];
+        auto_ptr<unsigned char> b(raw_b);
+		buffer = raw_b;
+        unsigned char *p = raw_b;
         int w_bytes = w;
         for (long y=0; y<h; ++y)
         {
@@ -241,7 +253,7 @@ namespace PyEncodedAttribute
                     else if (PyInt_Check(cell) || PyLong_Check(cell))
                     {
                         long byte = PyLong_AsLong(cell);
-                        if (byte==-1 and PyErr_Occurred())
+                        if (byte==-1 && PyErr_Occurred())
                         {
                             Py_DECREF(row);
                             Py_DECREF(cell);
@@ -277,6 +289,7 @@ namespace PyEncodedAttribute
             self.encode_gray16(buffer, w, h);
             return;
         }
+#ifndef DISABLE_PYTANGO_NUMPY
         else if (PyArray_Check(py_value_ptr))
         {
             w = PyArray_DIM(py_value_ptr, 1);
@@ -286,11 +299,14 @@ namespace PyEncodedAttribute
             self.encode_gray16(buffer, w, h);
             return;
         }
+#endif
         // It must be a py sequence
         // we are sure that w and h are given by python (see encoded_attribute.py)
-        unsigned short b[w*h];
-        buffer = b;
-        unsigned short *p = b;
+		const int length = w*h;
+        unsigned short *raw_b = new unsigned short[length];
+        auto_ptr<unsigned short> b(raw_b);
+		buffer = raw_b;
+        unsigned short *p = raw_b;
         int w_bytes = 2*w;
         for (long y=0; y<h; ++y)
         {
@@ -350,7 +366,7 @@ namespace PyEncodedAttribute
                     }
                     else if (PyInt_Check(cell) || PyLong_Check(cell))
                     {
-                        unsigned short word = PyLong_AsUnsignedLong(cell);
+                        unsigned short word = (unsigned short)PyLong_AsUnsignedLong(cell);
                         if (PyErr_Occurred())
                         {
                             Py_DECREF(row);
@@ -386,17 +402,21 @@ namespace PyEncodedAttribute
             self.encode_rgb24(buffer, w, h);
             return;
         }
+#ifndef DISABLE_PYTANGO_NUMPY
         else if (PyArray_Check(py_value_ptr))
         {
             buffer = static_cast<unsigned char*>(PyArray_DATA(py_value_ptr));
             self.encode_rgb24(buffer, w, h);
             return;
         }
+#endif
         // It must be a py sequence
         // we are sure that w and h are given by python (see encoded_attribute.py)
-        unsigned char b[w*h];
-        buffer = b;
-        unsigned char *p = b;
+		const int length = w*h;
+        unsigned char *raw_b = new unsigned char[length];
+        auto_ptr<unsigned char> b(raw_b);
+		buffer = raw_b;
+        unsigned char *p = raw_b;
         int w_bytes = 3*w;
         for (long y=0; y<h; ++y)
         {
@@ -459,23 +479,23 @@ namespace PyEncodedAttribute
                     else if (PyInt_Check(cell) || PyLong_Check(cell))
                     {
                         long byte = PyLong_AsLong(cell);
-                        if (byte==-1 and PyErr_Occurred())
+                        if (byte==-1 && PyErr_Occurred())
                         {
                             Py_DECREF(row);
                             Py_DECREF(cell);
                             boost::python::throw_error_already_set();
                         }
-                        if (BYTE_ORDER == LITTLE_ENDIAN)
+                        if (IS_BIGENDIAN())
                         {
-                            *p = (byte) & 0xFF; p++;
-                            *p = (byte >>  8) & 0xFF; p++;
-                            *p = (byte >> 16) & 0xFF; p++;
+                            *p = (unsigned char)(byte >> 16) & 0xFF; p++;
+                            *p = (unsigned char)(byte >>  8) & 0xFF; p++;
+                            *p = (unsigned char)(byte) & 0xFF; p++;
                         }
                         else
                         {
-                            *p = (byte >> 16) & 0xFF; p++;
-                            *p = (byte >>  8) & 0xFF; p++;
-                            *p = (byte) & 0xFF; p++;
+                            *p = (unsigned char)(byte) & 0xFF; p++;
+                            *p = (unsigned char)(byte >>  8) & 0xFF; p++;
+                            *p = (unsigned char)(byte >> 16) & 0xFF; p++;
                         }
                     }
                     Py_DECREF(cell);
@@ -496,17 +516,21 @@ namespace PyEncodedAttribute
             self.encode_jpeg_rgb24(buffer, w, h, quality);
             return;
         }
+#ifndef DISABLE_PYTANGO_NUMPY
         else if (PyArray_Check(py_value_ptr))
         {
             buffer = static_cast<unsigned char*>(PyArray_DATA(py_value_ptr));
             self.encode_jpeg_rgb24(buffer, w, h, quality);
             return;
         }
+#endif
         // It must be a py sequence
         // we are sure that w and h are given by python (see encoded_attribute.py)
-        unsigned char b[w*h];
-        buffer = b;
-        unsigned char *p = b;
+		const int length = w*h;
+        unsigned char *raw_b = new unsigned char[length];
+        auto_ptr<unsigned char> b(raw_b);
+		buffer = raw_b;
+        unsigned char *p = raw_b;
         int w_bytes = 3*w;
         for (long y=0; y<h; ++y)
         {
@@ -569,23 +593,23 @@ namespace PyEncodedAttribute
                     else if (PyInt_Check(cell) || PyLong_Check(cell))
                     {
                         long byte = PyLong_AsLong(cell);
-                        if (byte==-1 and PyErr_Occurred())
+                        if (byte==-1 && PyErr_Occurred())
                         {
                             Py_DECREF(row);
                             Py_DECREF(cell);
                             boost::python::throw_error_already_set();
                         }
-                        if (BYTE_ORDER == LITTLE_ENDIAN)
+                        if (IS_BIGENDIAN())
                         {
-                            *p = (byte) & 0xFF; p++;
-                            *p = (byte >>  8) & 0xFF; p++;
-                            *p = (byte >> 16) & 0xFF; p++;
+                            *p = (unsigned char)(byte >> 16) & 0xFF; p++;
+                            *p = (unsigned char)(byte >>  8) & 0xFF; p++;
+                            *p = (unsigned char)(byte) & 0xFF; p++;
                         }
                         else
                         {
-                            *p = (byte >> 16) & 0xFF; p++;
-                            *p = (byte >>  8) & 0xFF; p++;
-                            *p = (byte) & 0xFF; p++;
+                            *p = (unsigned char)(byte) & 0xFF; p++;
+                            *p = (unsigned char)(byte >>  8) & 0xFF; p++;
+                            *p = (unsigned char)(byte >> 16) & 0xFF; p++;
                         }
                     }
                     Py_DECREF(cell);
@@ -606,17 +630,21 @@ namespace PyEncodedAttribute
             self.encode_jpeg_rgb32(buffer, w, h, quality);
             return;
         }
+#ifndef DISABLE_PYTANGO_NUMPY
         else if (PyArray_Check(py_value_ptr))
         {
             buffer = static_cast<unsigned char*>(PyArray_DATA(py_value_ptr));
             self.encode_jpeg_rgb32(buffer, w, h, quality);
             return;
         }
+#endif
         // It must be a py sequence
         // we are sure that w and h are given by python (see encoded_attribute.py)
-        unsigned char b[w*h];
-        buffer = b;
-        unsigned char *p = b;
+		const int length = w*h;
+        unsigned char *raw_b = new unsigned char[length];
+        auto_ptr<unsigned char> b(raw_b);
+		buffer = raw_b;
+        unsigned char *p = raw_b;
         int w_bytes = 4*w;
         for (long y=0; y<h; ++y)
         {
@@ -680,25 +708,25 @@ namespace PyEncodedAttribute
                     else if (PyInt_Check(cell) || PyLong_Check(cell))
                     {
                         long byte = PyLong_AsLong(cell);
-                        if (byte==-1 and PyErr_Occurred())
+                        if (byte==-1 && PyErr_Occurred())
                         {
                             Py_DECREF(row);
                             Py_DECREF(cell);
                             boost::python::throw_error_already_set();
                         }
-                        if (BYTE_ORDER == LITTLE_ENDIAN)
+                        if (IS_BIGENDIAN())
                         {
-                            *p = (byte) & 0xFF; p++;
-                            *p = (byte >>  8) & 0xFF; p++;
-                            *p = (byte >> 16) & 0xFF; p++;
-                            *p = (byte >> 24) & 0xFF; p++;
+                            *p = (unsigned char)(byte >> 24) & 0xFF; p++;
+                            *p = (unsigned char)(byte >> 16) & 0xFF; p++;
+                            *p = (unsigned char)(byte >>  8) & 0xFF; p++;
+                            *p = (unsigned char)(byte) & 0xFF; p++;
                         }
                         else
                         {
-                            *p = (byte >> 24) & 0xFF; p++;
-                            *p = (byte >> 16) & 0xFF; p++;
-                            *p = (byte >>  8) & 0xFF; p++;
-                            *p = (byte) & 0xFF; p++;
+                            *p = (unsigned char)(byte) & 0xFF; p++;
+                            *p = (unsigned char)(byte >>  8) & 0xFF; p++;
+                            *p = (unsigned char)(byte >> 16) & 0xFF; p++;
+                            *p = (unsigned char)(byte >> 24) & 0xFF; p++;
                         }
                     }
                     Py_DECREF(cell);
@@ -856,6 +884,7 @@ namespace PyEncodedAttribute
         switch (extract_as)
         {
             case PyTango::ExtractAsNumpy:
+#ifndef DISABLE_PYTANGO_NUMPY
             {
                 npy_intp dims[2] = { height, width };
                 ret = PyArray_SimpleNewFromData(2, dims, NPY_USHORT, ch_ptr);
@@ -885,6 +914,7 @@ namespace PyEncodedAttribute
                 PyArray_BASE(ret) = guard;
                 break;
             }
+#endif
             case PyTango::ExtractAsString:
             {
                 ret = PyTuple_New(3);
@@ -992,6 +1022,7 @@ namespace PyEncodedAttribute
         switch (extract_as)
         {
             case PyTango::ExtractAsNumpy:
+#ifndef DISABLE_PYTANGO_NUMPY
             {
                 npy_intp dims[2] = { height, width };
                 ret = PyArray_SimpleNewFromData(2, dims, NPY_UINT32, ch_ptr);
@@ -1021,6 +1052,7 @@ namespace PyEncodedAttribute
                 PyArray_BASE(ret) = guard;
                 break;
             }
+#endif
             case PyTango::ExtractAsString:
             {
                 ret = PyTuple_New(3);
@@ -1069,8 +1101,16 @@ namespace PyEncodedAttribute
                     {
                         long idx = 4*(y*width+x);
                         // data comes in in big endian format
-                        uint32_t data;
-                        if (BYTE_ORDER == LITTLE_ENDIAN)
+						Tango::DevULong data;
+                        if (IS_BIGENDIAN())
+                        {
+                            char *p = reinterpret_cast<char *>(&data);
+                            *p = ch_ptr[idx++]; ++p;
+                            *p = ch_ptr[idx++]; ++p;
+                            *p = ch_ptr[idx++]; ++p;
+                            *p = ch_ptr[idx];
+                        }
+                        else
                         {
                             idx +=3;
                             char *p = reinterpret_cast<char *>(&data);
@@ -1079,14 +1119,6 @@ namespace PyEncodedAttribute
                             *p = ch_ptr[idx--]; ++p;
                             *p = ch_ptr[idx];
                         }
-                        else
-                        {
-                            char *p = reinterpret_cast<char *>(&data);
-                            *p = ch_ptr[idx++]; ++p;
-                            *p = ch_ptr[idx++]; ++p;
-                            *p = ch_ptr[idx++]; ++p;
-                            *p = ch_ptr[idx];
-                        }
                         PyTuple_SetItem(row, x, PyLong_FromUnsignedLong(data));
                     }
                     PyTuple_SetItem(ret, y, row);
@@ -1118,8 +1150,16 @@ namespace PyEncodedAttribute
                     {
                         long idx = 4*(y*width+x);
                         // data comes in in big endian format
-                        uint32_t data;
-                        if (BYTE_ORDER == LITTLE_ENDIAN)
+						Tango::DevULong data;
+                        if (IS_BIGENDIAN())
+                        {
+                            char *p = reinterpret_cast<char *>(&data);
+                            *p = ch_ptr[idx++]; ++p;
+                            *p = ch_ptr[idx++]; ++p;
+                            *p = ch_ptr[idx++]; ++p;
+                            *p = ch_ptr[idx];
+                        }
+                        else
                         {
                             idx +=3;
                             char *p = reinterpret_cast<char *>(&data);
@@ -1128,14 +1168,6 @@ namespace PyEncodedAttribute
                             *p = ch_ptr[idx--]; ++p;
                             *p = ch_ptr[idx];
                         }
-                        else
-                        {
-                            char *p = reinterpret_cast<char *>(&data);
-                            *p = ch_ptr[idx++]; ++p;
-                            *p = ch_ptr[idx++]; ++p;
-                            *p = ch_ptr[idx++]; ++p;
-                            *p = ch_ptr[idx];
-                        }
                         PyList_SetItem(row, x, PyLong_FromUnsignedLong(data));
                     }
                     PyList_SetItem(ret, y, row);
